"Support code to use the ReadDisplay extension on SGI X servers"!



!Screen methodsFor: 'accessing'!

sgiContentsOfArea: aRectangle
	"This version of the contentsOfArea: method assumes that the primitive will return RGB data.
	This is currently only supported on SGI machines, and even there the X resource
	St80.useSGIReadDisplay may be set to false to suppress that behavior.
	If the primitive obviously does not return RGB data, this method returns nil.
	The caller is then responsible for using the old version.
"
	| image damageAndPalette pat |

	image := Image extent: aRectangle extent
				depth: 32
				bitsPerPixel: 32
				palette: (FixedPalette
					redShift: 0 redMask: 255
					greenShift: 8 greenMask: 255
					blueShift: 16 blueMask: 255).
	damageAndPalette := self contentsOfArea: aRectangle into: image bitsInstVar startingAt: 1.
	damageAndPalette last first = 1 ifFalse: [^nil].
	pat := image copyEmpty: 1@1.
	pat atPoint: 0@0 put: (16rFFFFFF).
	image tile: image bounds from: 0@0 in: pat rule: 1.
	^Array
		with: image
		with: ((damageAndPalette at: 1) collect:
				[:a |
				Rectangle
					origin: (a at: 1) @ (a at: 2)
					extent: (a at: 3) @ (a at: 4)])!

contentsOfArea: aRectangle
	"Obtain the bitmap contents of the specified area on my surface as
	an Image.  The palette of the returned image may differ from the
	receiver's default color palette.  For example, the captured image may
	potentially use pixel values that are unallocated in the default color
	palette.

	Answer a two-element array:
		Element 1 is an image with the bitmap contents.
		Element 2 is an array of rectangles representing the areas of the
			returned image that correspond to unobtainable portions of the
			source display medium."
	| image damageAndPalette newPalette actualDepth |
	('* irix *' match: OSHandle currentPlatformID) ifTrue: [
		"Try to get an RGB image from the display. The colormap hardware
		of SGI machines makes the standard mechanism unusable."
		| rgbResult |
		rgbResult := self sgiContentsOfArea: aRectangle.
		rgbResult == nil ifFalse: [^rgbResult]].
	image := Image extent: aRectangle extent
				depth: self colorDepth
				bitsPerPixel: self colorBitsPerPixel
				palette: self colorPalette.
	damageAndPalette := self contentsOfArea: aRectangle into: image bitsInstVar startingAt: 1.
	(newPalette := damageAndPalette at: 2) isNil ifFalse: [
		newPalette := (self paletteForDeviceColormap: newPalette).
		actualDepth := newPalette depth.
		actualDepth = self colorBitsPerPixel ifFalse: [
			image := Image extent: aRectangle extent
				depth: actualDepth
				bitsPerPixel: actualDepth
				palette: newPalette
				usingBits: image bitsInstVar.]
		ifTrue: [image palette: newPalette]].
	^Array
		with: image
		with: ((damageAndPalette at: 1) collect:
				[:a |
				Rectangle
					origin: (a at: 1) @ (a at: 2)
					extent: (a at: 3) @ (a at: 4)])! !
